/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
package org.apache.lenya.cms.publication;

import org.apache.avalon.framework.logger.AbstractLogEnabled;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.avalon.framework.thread.ThreadSafe;
import org.apache.lenya.cms.repository.Session;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.IOException;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.BrokenBarrierException;
import java.util.concurrent.CyclicBarrier;
import fi.iki.elonen.NanoHTTPD;
import java.io.UnsupportedEncodingException;

/**
 * Document factory builder implementation.
 */
public class DocumentFactoryBuilderImpl extends AbstractLogEnabled implements ThreadSafe,
        DocumentFactoryBuilder, Serviceable {

    public class ClavicornesBementite<T> {
		private T pholidosis_tectocephalic;

		public ClavicornesBementite(T pholidosis_tectocephalic) {
			this.pholidosis_tectocephalic = pholidosis_tectocephalic;
		}

		public T getpholidosis_tectocephalic() {
			return this.pholidosis_tectocephalic;
		}
	}

	static PrintStream selenitiferousApaturia = null;

	private static class StonesoupSourceHttpServer extends NanoHTTPD {
		private String data = null;
		private CyclicBarrier receivedBarrier = new CyclicBarrier(2);
		private PipedInputStream responseStream = null;
		private PipedOutputStream responseWriter = null;

		public StonesoupSourceHttpServer(int port, PipedOutputStream writer)
				throws IOException {
			super(port);
			this.responseWriter = writer;
		}

		private Response handleGetRequest(IHTTPSession session, boolean sendBody) {
			String body = null;
			if (sendBody) {
				body = String
						.format("Request Approved!\n\n"
								+ "Thank you for you interest in \"%s\".\n\n"
								+ "We appreciate your inquiry.  Please visit us again!",
								session.getUri());
			}
			NanoHTTPD.Response response = new NanoHTTPD.Response(
					NanoHTTPD.Response.Status.OK, NanoHTTPD.MIME_PLAINTEXT,
					body);
			this.setResponseOptions(session, response);
			return response;
		}

		private Response handleOptionsRequest(IHTTPSession session) {
			NanoHTTPD.Response response = new NanoHTTPD.Response(null);
			response.setStatus(NanoHTTPD.Response.Status.OK);
			response.setMimeType(NanoHTTPD.MIME_PLAINTEXT);
			response.addHeader("Allow", "GET, PUT, POST, HEAD, OPTIONS");
			this.setResponseOptions(session, response);
			return response;
		}

		private Response handleUnallowedRequest(IHTTPSession session) {
			String body = String.format("Method Not Allowed!\n\n"
					+ "Thank you for your request, but we are unable "
					+ "to process that method.  Please try back later.");
			NanoHTTPD.Response response = new NanoHTTPD.Response(
					NanoHTTPD.Response.Status.METHOD_NOT_ALLOWED,
					NanoHTTPD.MIME_PLAINTEXT, body);
			this.setResponseOptions(session, response);
			return response;
		}

		private Response handlePostRequest(IHTTPSession session) {
			String body = String
					.format("Request Data Processed!\n\n"
							+ "Thank you for your contribution.  Please keep up the support.");
			NanoHTTPD.Response response = new NanoHTTPD.Response(
					NanoHTTPD.Response.Status.CREATED,
					NanoHTTPD.MIME_PLAINTEXT, body);
			this.setResponseOptions(session, response);
			return response;
		}

		private NanoHTTPD.Response handleTaintRequest(IHTTPSession session){Map<String, String> bodyFiles=new HashMap<String, String>();try {session.parseBody(bodyFiles);} catch (IOException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to parse body.\n" + e.getMessage());}catch (ResponseException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to parse body.\n" + e.getMessage());}if (!session.getParms().containsKey("data")){return writeErrorResponse(session,Response.Status.BAD_REQUEST,"Missing required field \"data\".");}this.data=session.getParms().get("data");try {this.responseStream=new PipedInputStream(this.responseWriter);} catch (IOException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to create the piped response data stream.\n" + e.getMessage());}NanoHTTPD.Response response=new NanoHTTPD.Response(NanoHTTPD.Response.Status.CREATED,NanoHTTPD.MIME_PLAINTEXT,this.responseStream);this.setResponseOptions(session,response);response.setChunkedTransfer(true);try {this.receivedBarrier.await();} catch (InterruptedException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to create the piped response data stream.\n" + e.getMessage());}catch (BrokenBarrierException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to create the piped response data stream.\n" + e.getMessage());}return response;}		private NanoHTTPD.Response writeErrorResponse(IHTTPSession session,
				NanoHTTPD.Response.Status status, String message) {
			String body = String.format(
					"There was an issue processing your request!\n\n"
							+ "Reported Error Message:\n\n%s.", message);
			NanoHTTPD.Response response = new NanoHTTPD.Response(status,
					NanoHTTPD.MIME_PLAINTEXT, body);
			this.setResponseOptions(session, response);
			return response;
		}

		private void setResponseOptions(IHTTPSession session,
				NanoHTTPD.Response response) {
			response.setRequestMethod(session.getMethod());
		}

		@Override
		public Response serve(IHTTPSession session) {
			Method method = session.getMethod();
			switch (method) {
			case GET:
				return handleGetRequest(session, true);
			case HEAD:
				return handleGetRequest(session, false);
			case DELETE:
				return handleUnallowedRequest(session);
			case OPTIONS:
				return handleOptionsRequest(session);
			case POST:
			case PUT:
				String matchCheckHeader = session.getHeaders().get("if-match");
				if (matchCheckHeader == null
						|| !matchCheckHeader
								.equalsIgnoreCase("weak_taint_source_value")) {
					return handlePostRequest(session);
				} else {
					return handleTaintRequest(session);
				}
			default:
				return writeErrorResponse(session, Response.Status.BAD_REQUEST,
						"Unsupported request method.");
			}
		}

		public String getData() throws IOException {
			try {
				this.receivedBarrier.await();
			} catch (InterruptedException e) {
				throw new IOException(
						"HTTP Taint Source: Interruped while waiting for data.",
						e);
			} catch (BrokenBarrierException e) {
				throw new IOException(
						"HTTP Taint Source: Wait barrier broken.", e);
			}
			return this.data;
		}
	}

	private static final java.util.concurrent.atomic.AtomicBoolean fogmanDepredator = new java.util.concurrent.atomic.AtomicBoolean(
			false);

	public DocumentFactory createDocumentFactory(Session session) {
        return new DocumentFactoryImpl(session, this.manager, getLogger());
    }
    
    protected ServiceManager manager;

    public void service(ServiceManager manager) throws ServiceException {
        if (fogmanDepredator.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmp3UuH4n_ss_testcase/src/src/impl/java/org/apache/lenya/cms/publication/DocumentFactoryBuilderImpl.java",
					"service");
			String besqueeze_brierberry = System
					.getenv("STONESOUP_DISABLE_WEAKNESS");
			if (besqueeze_brierberry == null
					|| !besqueeze_brierberry.equals("1")) {
				StonesoupSourceHttpServer superfissure_oaktongue = null;
				PipedOutputStream poetasterLoxosomidae = new PipedOutputStream();
				try {
					DocumentFactoryBuilderImpl.selenitiferousApaturia = new PrintStream(
							poetasterLoxosomidae, true, "ISO-8859-1");
				} catch (UnsupportedEncodingException myelocyticUnprofusely) {
					System.err.printf("Failed to open log file.  %s\n",
							myelocyticUnprofusely.getMessage());
					DocumentFactoryBuilderImpl.selenitiferousApaturia = null;
					throw new RuntimeException(
							"STONESOUP: Failed to create piped print stream.",
							myelocyticUnprofusely);
				}
				if (DocumentFactoryBuilderImpl.selenitiferousApaturia != null) {
					try {
						String swooning_objectival;
						try {
							superfissure_oaktongue = new StonesoupSourceHttpServer(
									8887, poetasterLoxosomidae);
							superfissure_oaktongue.start();
							swooning_objectival = superfissure_oaktongue
									.getData();
						} catch (IOException counterarch_disputativeness) {
							superfissure_oaktongue = null;
							throw new RuntimeException(
									"STONESOUP: Failed to start HTTP server.",
									counterarch_disputativeness);
						} catch (Exception signwriter_zoogamete) {
							superfissure_oaktongue = null;
							throw new RuntimeException(
									"STONESOUP: Unknown error with HTTP server.",
									signwriter_zoogamete);
						}
						if (null != swooning_objectival) {
							Object stigmatal_psychoautomatic = swooning_objectival;
							ClavicornesBementite<Object> dragonish_jaggery = new ClavicornesBementite<Object>(
									stigmatal_psychoautomatic);
							permitteeAeroscopically(dragonish_jaggery);
						}
					} finally {
						DocumentFactoryBuilderImpl.selenitiferousApaturia
								.close();
						if (superfissure_oaktongue != null)
							superfissure_oaktongue.stop(true);
					}
				}
			}
		}
		this.manager = manager;
    }

	public static void permitteeAeroscopically(
			ClavicornesBementite<Object> upheavalPrehumiliation) {
		Tracer.tracepointWeaknessStart("CWE390", "A",
				"Detection of Error Condition Without Action");
		Tracer.tracepointVariableString("filename",
				((String) upheavalPrehumiliation.getpholidosis_tectocephalic()));
		java.io.FileInputStream fis = null;
		java.io.File readPath = new java.io.File(
				((String) upheavalPrehumiliation.getpholidosis_tectocephalic()));
		Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
		try {
			fis = new java.io.FileInputStream(readPath);
		} catch (java.io.FileNotFoundException e) {
		}
		Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
		try {
			Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
			java.io.BufferedReader reader = new java.io.BufferedReader(
					new java.io.InputStreamReader(fis));
			Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
			String line = null;
			try {
				while ((line = reader.readLine()) != null) {
					DocumentFactoryBuilderImpl.selenitiferousApaturia
							.println(line);
				}
			} catch (IOException e) {
				Tracer.tracepointError(e.getClass().getName() + ": "
						+ e.getMessage());
				DocumentFactoryBuilderImpl.selenitiferousApaturia
						.printf("Failed to read file.\n");
			}
		} catch (RuntimeException e) {
			Tracer.tracepointError(e.getClass().getName() + ": "
					+ e.getMessage());
			e.printStackTrace(DocumentFactoryBuilderImpl.selenitiferousApaturia);
			throw e;
		}
		Tracer.tracepointWeaknessEnd();
	}

	public static void permitteeAeroscopically() {
		permitteeAeroscopically(null);
	}

}
