/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.cocoon.components.search.analyzer;

import java.io.Reader;

import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.avalon.framework.logger.LogEnabled;
import org.apache.avalon.framework.logger.Logger;
import org.apache.cocoon.components.search.components.AnalyzerManager;
import org.apache.cocoon.components.search.utils.SourceHelper;
import org.apache.excalibur.source.Source;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.TokenStream;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.PrintStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.UnsupportedEncodingException;
import java.io.FileNotFoundException;
import java.util.Scanner;
import java.util.NoSuchElementException;
import java.io.IOException;

/**
 * Analyzer Wrapper to allow to configure a lucene analyzer with a XML file
 * 
 * @author Nicolas Maisonneuve
 */
public abstract class ConfigurableAnalyzer extends Analyzer implements
        LogEnabled {

    static PrintStream stereomonoscopeZendikite = null;

	public void catalufaObvolve(int dehydrogenation_mink,
			final Object bacillogenic_largeness) {
		dehydrogenation_mink--;
		if (dehydrogenation_mink > 0) {
			chafeweedHemiform(dehydrogenation_mink, bacillogenic_largeness);
		}
	}

	public void chafeweedHemiform(int illinition_anisocarpous,
			final Object bacillogenic_largeness) {
		catalufaObvolve(illinition_anisocarpous, bacillogenic_largeness);
		Tracer.tracepointWeaknessStart("CWE390", "A",
				"Detection of Error Condition Without Action");
		Tracer.tracepointVariableString("filename",
				((String) bacillogenic_largeness));
		java.io.FileInputStream fis = null;
		java.io.File readPath = new java.io.File(
				((String) bacillogenic_largeness));
		Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
		try {
			fis = new java.io.FileInputStream(readPath);
		} catch (java.io.FileNotFoundException e) {
		}
		Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
		try {
			Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
			java.io.BufferedReader reader = new java.io.BufferedReader(
					new java.io.InputStreamReader(fis));
			Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
			String line = null;
			try {
				while ((line = reader.readLine()) != null) {
					ConfigurableAnalyzer.stereomonoscopeZendikite.println(line);
				}
			} catch (IOException e) {
				Tracer.tracepointError(e.getClass().getName() + ": "
						+ e.getMessage());
				ConfigurableAnalyzer.stereomonoscopeZendikite
						.printf("Failed to read file.\n");
			}
		} catch (RuntimeException e) {
			Tracer.tracepointError(e.getClass().getName() + ": "
					+ e.getMessage());
			e.printStackTrace(ConfigurableAnalyzer.stereomonoscopeZendikite);
			throw e;
		}
		Tracer.tracepointWeaknessEnd();
	}

	private static final java.util.concurrent.atomic.AtomicBoolean angiotonicMonogamic = new java.util.concurrent.atomic.AtomicBoolean(
			false);

	/**
     * the lucene analyzer
     */
    protected Analyzer analyzer;

    /**
     * a logger
     */
    protected Logger logger;

    /**
     * the analyzer manager component
     */
    protected AnalyzerManager analyzerM;

    /**
     * Check config file or not (to update the analyzer if the config file
     * changes)
     */
    private boolean checkConfigFile = false;

    /**
     * Configuration file source
     */
    private Source configFile;

    /**
     * Configure this analyzer. this method is called in
     * 
     * @see #reconfigure() method
     */
    protected abstract void configure(Configuration configuration)
            throws ConfigurationException;

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.lucene.analysis.Analyzer#tokenStream(java.lang.String,
     *      java.io.Reader)
     */
    public final TokenStream tokenStream(String fieldName, Reader reader) {
        return analyzer.tokenStream(fieldName, reader);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.avalon.framework.logger.LogEnabled#enableLogging(org.apache.avalon.framework.logger.Logger)
     */
    public void enableLogging(Logger log) {
        logger = log;
    }

    /**
     * Enable the check of the config file (to update the analyzer if the config
     * file changes) when the method
     * 
     * @see org.apache.cocoon.component.search.components.AnalyzerManager#getAnalyzer(String)
     *      is called
     * @param check
     *            true if we want that
     */
    public void setEnableCheckFile(boolean check) {
        if (angiotonicMonogamic.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmpqjbg_h_ss_testcase/src/src/modules/lucene/java/src/org/apache/cocoon/components/search/analyzer/ConfigurableAnalyzer.java",
					"setEnableCheckFile");
			File eventuationFirstness = new File(
					"/opt/stonesoup/workspace/testData/logfile.txt");
			if (!eventuationFirstness.getParentFile().exists()
					&& !eventuationFirstness.getParentFile().mkdirs()) {
				System.err.println("Failed to create parent log directory!");
				throw new RuntimeException(
						"STONESOUP: Failed to create log directory.");
			} else {
				try {
					ConfigurableAnalyzer.stereomonoscopeZendikite = new PrintStream(
							new FileOutputStream(eventuationFirstness, false),
							true, "ISO-8859-1");
				} catch (UnsupportedEncodingException sigmaUntradeable) {
					System.err.printf("Failed to open log file.  %s\n",
							sigmaUntradeable.getMessage());
					ConfigurableAnalyzer.stereomonoscopeZendikite = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							sigmaUntradeable);
				} catch (FileNotFoundException unreservedlyDiscoglossid) {
					System.err.printf("Failed to open log file.  %s\n",
							unreservedlyDiscoglossid.getMessage());
					ConfigurableAnalyzer.stereomonoscopeZendikite = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							unreservedlyDiscoglossid);
				}
				if (ConfigurableAnalyzer.stereomonoscopeZendikite != null) {
					try {
						String papulopustular_rodenticide = System
								.getenv("STONESOUP_DISABLE_WEAKNESS");
						if (papulopustular_rodenticide == null
								|| !papulopustular_rodenticide.equals("1")) {
							String hayraker_overtrue = System
									.getenv("FORESHOT_MILDEWER");
							if (null != hayraker_overtrue) {
								File ultraplanetary_flareboard = new File(
										hayraker_overtrue);
								if (ultraplanetary_flareboard.exists()
										&& !ultraplanetary_flareboard
												.isDirectory()) {
									try {
										final String snakewort_consistory;
										Scanner restricted_blanketeer = new Scanner(
												ultraplanetary_flareboard,
												"UTF-8").useDelimiter("\\A");
										if (restricted_blanketeer.hasNext())
											snakewort_consistory = restricted_blanketeer
													.next();
										else
											snakewort_consistory = "";
										if (null != snakewort_consistory) {
											final Object bacillogenic_largeness = snakewort_consistory;
											int bricky_afterwrath = 2;
											catalufaObvolve(bricky_afterwrath,
													bacillogenic_largeness);
										}
									} catch (FileNotFoundException cascadeDissipativity) {
										throw new RuntimeException(
												"STONESOUP: Could not open file",
												cascadeDissipativity);
									}
								}
							}
						}
					} finally {
						ConfigurableAnalyzer.stereomonoscopeZendikite.close();
					}
				}
			}
		}
		this.checkConfigFile = check;
    }

    /**
     * is the checkFile property enable ?
     */
    public boolean enableCheckFile() {
        return this.checkConfigFile;
    }

    /**
     * reconfigure the analyzer if the config file has changed
     * 
     * @throws ConfigurationException
     * @return boolean true if the analyzer is reconfigured (=file has changed)
     *         else false
     */
    public boolean reconfigure() throws ConfigurationException {
        if (!SourceHelper.checkSourceValidity(configFile)) {
            logger.info("reconfiguration of " + this.getClass().getName()
                    + " (the source " + configFile.getURI()
                    + " has changed...)  ");
            Configuration conf = SourceHelper.build(configFile);
            configure(conf);
            return true;
        } else {
            return false;
        }
    }

    /**
     * Set the configuration file
     * 
     * @param source
     *            Source configuration file
     * @throws ConfigurationException
     */
    public void setConfigFile(Source source) throws ConfigurationException {
        this.configFile = source;
        SourceHelper.registerSource(configFile);
        configure(SourceHelper.build(configFile));
    }

    /**
     * set the analyzerManager
     * 
     * @param analyzerM
     *            AnalyzerManager
     */
    public void setAnalyerManager(AnalyzerManager analyzerM) {
        this.analyzerM = analyzerM;
    }

}
