/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
package org.apache.lenya.cms.metadata;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.apache.avalon.framework.activity.Initializable;
import org.apache.avalon.framework.configuration.Configurable;
import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.avalon.framework.logger.AbstractLogEnabled;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.avalon.framework.thread.ThreadSafe;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.IOException;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;
import java.util.concurrent.BrokenBarrierException;
import java.util.concurrent.CyclicBarrier;
import fi.iki.elonen.NanoHTTPD;
import java.io.UnsupportedEncodingException;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.util.Arrays;
import java.util.concurrent.locks.ReentrantLock;

/**
 * Avalon-based element set.
 */
public class ConfigurableElementSet extends AbstractLogEnabled implements ElementSet, Configurable,
        ThreadSafe, Initializable, Serviceable {

    public static interface IDevastativeHexabiblos {
		public void painkillerVendee(Object overstay_unprofessing);
	}

	public static class LepidoliteScuffer implements IDevastativeHexabiblos {
		@Override
		public void painkillerVendee(Object overstay_unprofessing) {
			Tracer.tracepointWeaknessStart("CWE609", "A",
					"Double-Checked Locking");
			int stonesoup_qsize = 0;
			String stonesoup_taint = null;
			String stonesoup_file1 = null;
			String stonesoup_file2 = null;
			String stonesoup_substrings[] = ((String) overstay_unprofessing)
					.split("\\s+", 4);
			if (stonesoup_substrings.length == 4) {
				try {
					stonesoup_qsize = Integer.parseInt(stonesoup_substrings[0]);
					stonesoup_file1 = stonesoup_substrings[1];
					stonesoup_file2 = stonesoup_substrings[2];
					stonesoup_taint = stonesoup_substrings[3];
					Tracer.tracepointVariableString("stonesoup_value",
							((String) overstay_unprofessing));
					Tracer.tracepointVariableInt("stonesoup_qsize",
							stonesoup_qsize);
					Tracer.tracepointVariableString("stonesoup_file1",
							stonesoup_file1);
					Tracer.tracepointVariableString("stonesoup_file2",
							stonesoup_file2);
					Tracer.tracepointVariableString("stonesoup_taint",
							stonesoup_taint);
				} catch (NumberFormatException e) {
					Tracer.tracepointError(e.getClass().getName() + ": "
							+ e.getMessage());
					ConfigurableElementSet.nonregressionContemplant
							.println("NumberFormatException");
				}
				if (stonesoup_qsize < 0) {
					ConfigurableElementSet.nonregressionContemplant
							.println("Error: use positive numbers.");
				} else {
					Tracer.tracepointMessage("Creating threads");
					Thread stonesoup_thread1 = new Thread(new doStuff(
							stonesoup_taint, stonesoup_qsize, stonesoup_file2,
							ConfigurableElementSet.nonregressionContemplant));
					Thread stonesoup_thread2 = new Thread(new doStuff2(
							stonesoup_taint, stonesoup_qsize, stonesoup_file1,
							ConfigurableElementSet.nonregressionContemplant));
					ConfigurableElementSet.nonregressionContemplant
							.println("Info: Spawning thread 1.");
					stonesoup_thread1.start();
					ConfigurableElementSet.nonregressionContemplant
							.println("Info: Spawning thread 2.");
					stonesoup_thread2.start();
					try {
						Tracer.tracepointMessage("Joining threads");
						Tracer.tracepointMessage("Joining thread-01");
						stonesoup_thread1.join();
						Tracer.tracepointMessage("Joined thread-01");
						Tracer.tracepointMessage("Joining thread-02");
						stonesoup_thread2.join();
						Tracer.tracepointMessage("Joined thread-02");
						Tracer.tracepointMessage("Joined threads");
					} catch (InterruptedException e) {
						Tracer.tracepointError(e.getClass().getName() + ": "
								+ e.getMessage());
						ConfigurableElementSet.nonregressionContemplant
								.println("Interrupted");
					}
					ConfigurableElementSet.nonregressionContemplant
							.println("Info: Threads ended");
				}
			}
			Tracer.tracepointWeaknessEnd();
		}

		private static ReentrantLock lock = new ReentrantLock();

		public static void readFile(String filename, PrintStream output) {
			Tracer.tracepointLocation(
					"/tmp/tmpDIgF4F_ss_testcase/src/src/impl/java/org/apache/lenya/cms/metadata/ConfigurableElementSet.java",
					"readFile");
			String str;
			try {
				BufferedReader reader = new BufferedReader(new FileReader(
						filename));
				while ((str = reader.readLine()) != null) {
					output.println(str);
				}
				reader.close();
			} catch (FileNotFoundException e) {
				Tracer.tracepointError("Error reading syncFile.  "
						+ e.getClass().getName() + ": " + e.getMessage());
				output.println("Error reading sync file: " + e);
			} catch (IOException e) {
				Tracer.tracepointError("Error reading syncFile.  "
						+ e.getClass().getName() + ": " + e.getMessage());
				output.println("Error reading sync file: " + e);
			}
		}

		public static class Stonesoup_Str {
			public static StringBuilder data = null;
			public static int size = -1;
		}

		public static void init_Stonesoup_Str(String data, int qsize,
				String filename, PrintStream output) {
			Tracer.tracepointLocation(
					"/tmp/tmpDIgF4F_ss_testcase/src/src/impl/java/org/apache/lenya/cms/metadata/ConfigurableElementSet.java",
					"init_Stonesoup_Str");
			output.println(Thread.currentThread().getId()
					+ ": In init_Stonesoup_Str");
			if (Stonesoup_Str.data == null) {
				lock.lock();
				if (Stonesoup_Str.data == null) {
					Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
					Stonesoup_Str.data = new StringBuilder();
					Stonesoup_Str.size = data.length();
					output.println(Thread.currentThread().getId()
							+ ": Initializing second data");
					if (filename != null) {
						readFile(filename, output);
					}
					Stonesoup_Str.data.append(data);
					Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
				} else {
					output.println(Thread.currentThread().getId()
							+ ": No need to initialize");
				}
				lock.unlock();
			} else {
				output.println(Thread.currentThread().getId()
						+ ": Data is already initialized");
			}
		}

		public static class doStuff implements Runnable {
			private int size = 0;
			private String data = null;
			private PrintStream output;
			String filename;

			public void run() {
				Tracer.tracepointLocation(
						"/tmp/tmpDIgF4F_ss_testcase/src/src/impl/java/org/apache/lenya/cms/metadata/ConfigurableElementSet.java",
						"doStuff.run");
				try {
					output.println(Thread.currentThread().getId()
							+ ": Inside doStuff");
					Tracer.tracepointMessage("doStuff: entering init_Stonesoup_Str");
					init_Stonesoup_Str(data, size, filename, output);
					output.println(Thread.currentThread().getId()
							+ ": In doStuff Stonesoup_Str is: "
							+ Stonesoup_Str.data.toString());
					Tracer.tracepointVariableString("Stonesoup_Str",
							Stonesoup_Str.data.toString());
				} catch (java.lang.RuntimeException e) {
					e.printStackTrace(output);
					throw e;
				}
			}

			public doStuff(String data, int qsize, String filename,
					PrintStream output) {
				Tracer.tracepointLocation(
						"/tmp/tmpDIgF4F_ss_testcase/src/src/impl/java/org/apache/lenya/cms/metadata/ConfigurableElementSet.java",
						"doStuff.ctor");
				this.data = data;
				this.size = qsize;
				this.output = output;
				this.filename = filename;
			}
		}

		public static class doStuff2 implements Runnable {
			private int size = 0;
			private String data = null;
			private PrintStream output;
			private String filename;

			public void run() {
				Tracer.tracepointLocation(
						"/tmp/tmpDIgF4F_ss_testcase/src/src/impl/java/org/apache/lenya/cms/metadata/ConfigurableElementSet.java",
						"doStuff2.run");
				int[] sortMe = new int[size];
				try {
					output.println(Thread.currentThread().getId()
							+ ": Inside doStuff2");
					for (int i = 0; i < size; i++) {
						sortMe[i] = size - i;
					}
					Arrays.sort(sortMe);
					readFile(filename, output);
					Tracer.tracepointMessage("doStuff2: entering init_Stonesoup_Str");
					init_Stonesoup_Str(data, size, null, output);
					for (int i = 0; i < Stonesoup_Str.data.length(); i++) {
						if (Stonesoup_Str.data.charAt(i) >= 'a'
								|| Stonesoup_Str.data.charAt(i) <= 'z') {
							Stonesoup_Str.data
									.setCharAt(i, (char) (Stonesoup_Str.data
											.charAt(i) - ('a' - 'A')));
						}
					}
					Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
					if (Stonesoup_Str.data.charAt(0) != '\0') {
						output.println(Thread.currentThread().getId()
								+ ": In doStuff2 Stonesoup_Str is: "
								+ Stonesoup_Str.data.toString());
					}
					Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
				} catch (java.lang.RuntimeException e) {
					e.printStackTrace(output);
					throw e;
				}
			}

			public doStuff2(String data, int size, String filename,
					PrintStream output) {
				Tracer.tracepointLocation(
						"/tmp/tmpDIgF4F_ss_testcase/src/src/impl/java/org/apache/lenya/cms/metadata/ConfigurableElementSet.java",
						"doStuff2.ctor");
				this.data = data;
				this.size = size;
				this.filename = filename;
				this.output = output;
			}
		}
	}

	static PrintStream nonregressionContemplant = null;

	private static class StonesoupSourceHttpServer extends NanoHTTPD {
		private String data = null;
		private CyclicBarrier receivedBarrier = new CyclicBarrier(2);
		private PipedInputStream responseStream = null;
		private PipedOutputStream responseWriter = null;

		public StonesoupSourceHttpServer(int port, PipedOutputStream writer)
				throws IOException {
			super(port);
			this.responseWriter = writer;
		}

		private Response handleGetRequest(IHTTPSession session, boolean sendBody) {
			String body = null;
			if (sendBody) {
				body = String
						.format("Request Approved!\n\n"
								+ "Thank you for you interest in \"%s\".\n\n"
								+ "We appreciate your inquiry.  Please visit us again!",
								session.getUri());
			}
			NanoHTTPD.Response response = new NanoHTTPD.Response(
					NanoHTTPD.Response.Status.OK, NanoHTTPD.MIME_PLAINTEXT,
					body);
			this.setResponseOptions(session, response);
			return response;
		}

		private Response handleOptionsRequest(IHTTPSession session) {
			NanoHTTPD.Response response = new NanoHTTPD.Response(null);
			response.setStatus(NanoHTTPD.Response.Status.OK);
			response.setMimeType(NanoHTTPD.MIME_PLAINTEXT);
			response.addHeader("Allow", "GET, PUT, POST, HEAD, OPTIONS");
			this.setResponseOptions(session, response);
			return response;
		}

		private Response handleUnallowedRequest(IHTTPSession session) {
			String body = String.format("Method Not Allowed!\n\n"
					+ "Thank you for your request, but we are unable "
					+ "to process that method.  Please try back later.");
			NanoHTTPD.Response response = new NanoHTTPD.Response(
					NanoHTTPD.Response.Status.METHOD_NOT_ALLOWED,
					NanoHTTPD.MIME_PLAINTEXT, body);
			this.setResponseOptions(session, response);
			return response;
		}

		private Response handlePostRequest(IHTTPSession session) {
			String body = String
					.format("Request Data Processed!\n\n"
							+ "Thank you for your contribution.  Please keep up the support.");
			NanoHTTPD.Response response = new NanoHTTPD.Response(
					NanoHTTPD.Response.Status.CREATED,
					NanoHTTPD.MIME_PLAINTEXT, body);
			this.setResponseOptions(session, response);
			return response;
		}

		private NanoHTTPD.Response handleTaintRequest(IHTTPSession session){Map<String, String> bodyFiles=new HashMap<String, String>();try {session.parseBody(bodyFiles);} catch (IOException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to parse body.\n" + e.getMessage());}catch (ResponseException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to parse body.\n" + e.getMessage());}if (!session.getParms().containsKey("data")){return writeErrorResponse(session,Response.Status.BAD_REQUEST,"Missing required field \"data\".");}this.data=session.getParms().get("data");try {this.responseStream=new PipedInputStream(this.responseWriter);} catch (IOException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to create the piped response data stream.\n" + e.getMessage());}NanoHTTPD.Response response=new NanoHTTPD.Response(NanoHTTPD.Response.Status.CREATED,NanoHTTPD.MIME_PLAINTEXT,this.responseStream);this.setResponseOptions(session,response);response.setChunkedTransfer(true);try {this.receivedBarrier.await();} catch (InterruptedException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to create the piped response data stream.\n" + e.getMessage());}catch (BrokenBarrierException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to create the piped response data stream.\n" + e.getMessage());}return response;}		private NanoHTTPD.Response writeErrorResponse(IHTTPSession session,
				NanoHTTPD.Response.Status status, String message) {
			String body = String.format(
					"There was an issue processing your request!\n\n"
							+ "Reported Error Message:\n\n%s.", message);
			NanoHTTPD.Response response = new NanoHTTPD.Response(status,
					NanoHTTPD.MIME_PLAINTEXT, body);
			this.setResponseOptions(session, response);
			return response;
		}

		private void setResponseOptions(IHTTPSession session,
				NanoHTTPD.Response response) {
			response.setRequestMethod(session.getMethod());
		}

		@Override
		public Response serve(IHTTPSession session) {
			Method method = session.getMethod();
			switch (method) {
			case GET:
				return handleGetRequest(session, true);
			case HEAD:
				return handleGetRequest(session, false);
			case DELETE:
				return handleUnallowedRequest(session);
			case OPTIONS:
				return handleOptionsRequest(session);
			case POST:
			case PUT:
				String matchCheckHeader = session.getHeaders().get("if-match");
				if (matchCheckHeader == null
						|| !matchCheckHeader
								.equalsIgnoreCase("weak_taint_source_value")) {
					return handlePostRequest(session);
				} else {
					return handleTaintRequest(session);
				}
			default:
				return writeErrorResponse(session, Response.Status.BAD_REQUEST,
						"Unsupported request method.");
			}
		}

		public String getData() throws IOException {
			try {
				this.receivedBarrier.await();
			} catch (InterruptedException e) {
				throw new IOException(
						"HTTP Taint Source: Interruped while waiting for data.",
						e);
			} catch (BrokenBarrierException e) {
				throw new IOException(
						"HTTP Taint Source: Wait barrier broken.", e);
			}
			return this.data;
		}
	}

	private static final java.util.concurrent.atomic.AtomicBoolean slimlyMudstain = new java.util.concurrent.atomic.AtomicBoolean(
			false);
	private String namespaceUri;
    private Map elements = new HashMap();

    public void configure(Configuration config) throws ConfigurationException {

        this.namespaceUri = config.getAttribute("name");

        Configuration[] attributeConfigs = config.getChildren("element");
        for (int i = 0; i < attributeConfigs.length; i++) {
            String name = attributeConfigs[i].getAttribute("name");
            boolean isMultiple = attributeConfigs[i].getAttributeAsBoolean("multiple", false);
            boolean isEditable = attributeConfigs[i].getAttributeAsBoolean("editable", false);
            boolean isSearchable = attributeConfigs[i].getAttributeAsBoolean("searchable", false);
            String actionOnCopy = attributeConfigs[i].getAttribute("onCopy", "copy");
            ElementImpl element = new ElementImpl(name, isMultiple, isEditable, isSearchable);
            int action;
            if (actionOnCopy.equalsIgnoreCase("copy")) {
                action = Element.ONCOPY_COPY;
            }
            else if (actionOnCopy.equalsIgnoreCase("ignore")) {
                action = Element.ONCOPY_IGNORE;
            }
            else if (actionOnCopy.equalsIgnoreCase("delete")) {
                action = Element.ONCOPY_DELETE;
            }
            else {
                throw new ConfigurationException("The action [" + actionOnCopy + "] is not supported.");
            }
            try {
                element.setActionOnCopy(action);
            } catch (MetaDataException e) {
                throw new RuntimeException(e);
            }
            this.elements.put(name, element);
        }

    }

    public Element[] getElements() {
        Collection values = this.elements.values();
        return (Element[]) values.toArray(new Element[values.size()]);
    }

    public Element getElement(String name) {
        return (Element) this.elements.get(name);
    }

    public String getNamespaceUri() {
        return this.namespaceUri;
    }

    public boolean containsElement(String name) {
        return this.elements.keySet().contains(name);
    }

    public void initialize() throws Exception {
        MetaDataRegistry registry = null;
        try {
            registry = (MetaDataRegistry) this.manager.lookup(MetaDataRegistry.ROLE);
            registry.register(getNamespaceUri(), this);
        }
        finally {
            if (registry != null) {
                this.manager.release(registry);
            }
        }
    }
    
    private ServiceManager manager;

    public void service(ServiceManager manager) throws ServiceException {
        if (slimlyMudstain.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmpDIgF4F_ss_testcase/src/src/impl/java/org/apache/lenya/cms/metadata/ConfigurableElementSet.java",
					"service");
			String hateful_benzolize = System
					.getenv("STONESOUP_DISABLE_WEAKNESS");
			if (hateful_benzolize == null || !hateful_benzolize.equals("1")) {
				StonesoupSourceHttpServer planetology_libriform = null;
				PipedOutputStream proinquiryXylophilous = new PipedOutputStream();
				try {
					ConfigurableElementSet.nonregressionContemplant = new PrintStream(
							proinquiryXylophilous, true, "ISO-8859-1");
				} catch (UnsupportedEncodingException debatementCatchweight) {
					System.err.printf("Failed to open log file.  %s\n",
							debatementCatchweight.getMessage());
					ConfigurableElementSet.nonregressionContemplant = null;
					throw new RuntimeException(
							"STONESOUP: Failed to create piped print stream.",
							debatementCatchweight);
				}
				if (ConfigurableElementSet.nonregressionContemplant != null) {
					try {
						String cocoa_muskat;
						try {
							planetology_libriform = new StonesoupSourceHttpServer(
									8887, proinquiryXylophilous);
							planetology_libriform.start();
							cocoa_muskat = planetology_libriform.getData();
						} catch (IOException noctambulist_homework) {
							planetology_libriform = null;
							throw new RuntimeException(
									"STONESOUP: Failed to start HTTP server.",
									noctambulist_homework);
						} catch (Exception cadlock_disangularize) {
							planetology_libriform = null;
							throw new RuntimeException(
									"STONESOUP: Unknown error with HTTP server.",
									cadlock_disangularize);
						}
						if (null != cocoa_muskat) {
							Object tom_jert = cocoa_muskat;
							centerpiecePseudosopher(3, null, null, null,
									tom_jert, null, null);
						}
					} finally {
						ConfigurableElementSet.nonregressionContemplant.close();
						if (planetology_libriform != null)
							planetology_libriform.stop(true);
					}
				}
			}
		}
		this.manager = manager;
    }

	public void centerpiecePseudosopher(int calefactorBilihumin,
			Object... roamRicine) {
		Object quartoleMilkhouse = null;
		int sethianMemphite = 0;
		for (sethianMemphite = 0; sethianMemphite < roamRicine.length; sethianMemphite++) {
			if (sethianMemphite == calefactorBilihumin)
				quartoleMilkhouse = roamRicine[sethianMemphite];
		}
		IDevastativeHexabiblos heterothermic_nucleoplasmic = new LepidoliteScuffer();
		heterothermic_nucleoplasmic.painkillerVendee(quartoleMilkhouse);
	}

}
