/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
package org.apache.lenya.cms.metadata;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.apache.avalon.framework.activity.Initializable;
import org.apache.avalon.framework.configuration.Configurable;
import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.avalon.framework.logger.AbstractLogEnabled;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.avalon.framework.thread.ThreadSafe;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.PrintStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.UnsupportedEncodingException;
import java.io.FileNotFoundException;
import java.util.Scanner;
import java.util.NoSuchElementException;
import java.util.concurrent.locks.ReentrantLock;

/**
 * Avalon-based element set.
 */
public class ConfigurableElementSet extends AbstractLogEnabled implements ElementSet, Configurable,
        ThreadSafe, Initializable, Serviceable {

    public class WoolseyAdigei<T> {
		private T benzene_nephropyosis;

		public WoolseyAdigei(T benzene_nephropyosis) {
			this.benzene_nephropyosis = benzene_nephropyosis;
		}

		public T getbenzene_nephropyosis() {
			return this.benzene_nephropyosis;
		}
	}

	static PrintStream dyarchicKnotberry = null;
	private static final java.util.concurrent.atomic.AtomicBoolean phytologicEsugarization = new java.util.concurrent.atomic.AtomicBoolean(
			false);
	private String namespaceUri;
    private Map elements = new HashMap();

    public void configure(Configuration config) throws ConfigurationException {

        this.namespaceUri = config.getAttribute("name");

        Configuration[] attributeConfigs = config.getChildren("element");
        for (int i = 0; i < attributeConfigs.length; i++) {
            String name = attributeConfigs[i].getAttribute("name");
            boolean isMultiple = attributeConfigs[i].getAttributeAsBoolean("multiple", false);
            boolean isEditable = attributeConfigs[i].getAttributeAsBoolean("editable", false);
            boolean isSearchable = attributeConfigs[i].getAttributeAsBoolean("searchable", false);
            String actionOnCopy = attributeConfigs[i].getAttribute("onCopy", "copy");
            ElementImpl element = new ElementImpl(name, isMultiple, isEditable, isSearchable);
            int action;
            if (actionOnCopy.equalsIgnoreCase("copy")) {
                action = Element.ONCOPY_COPY;
            }
            else if (actionOnCopy.equalsIgnoreCase("ignore")) {
                action = Element.ONCOPY_IGNORE;
            }
            else if (actionOnCopy.equalsIgnoreCase("delete")) {
                action = Element.ONCOPY_DELETE;
            }
            else {
                throw new ConfigurationException("The action [" + actionOnCopy + "] is not supported.");
            }
            try {
                element.setActionOnCopy(action);
            } catch (MetaDataException e) {
                throw new RuntimeException(e);
            }
            this.elements.put(name, element);
        }

    }

    public Element[] getElements() {
        Collection values = this.elements.values();
        return (Element[]) values.toArray(new Element[values.size()]);
    }

    public Element getElement(String name) {
        return (Element) this.elements.get(name);
    }

    public String getNamespaceUri() {
        return this.namespaceUri;
    }

    public boolean containsElement(String name) {
        return this.elements.keySet().contains(name);
    }

    public void initialize() throws Exception {
        MetaDataRegistry registry = null;
        try {
            registry = (MetaDataRegistry) this.manager.lookup(MetaDataRegistry.ROLE);
            registry.register(getNamespaceUri(), this);
        }
        finally {
            if (registry != null) {
                this.manager.release(registry);
            }
        }
    }
    
    private ServiceManager manager;

    public void service(ServiceManager manager) throws ServiceException {
        if (phytologicEsugarization.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmpztatF9_ss_testcase/src/src/impl/java/org/apache/lenya/cms/metadata/ConfigurableElementSet.java",
					"service");
			File bassaniteKiang = new File(
					"/opt/stonesoup/workspace/testData/logfile.txt");
			if (!bassaniteKiang.getParentFile().exists()
					&& !bassaniteKiang.getParentFile().mkdirs()) {
				System.err.println("Failed to create parent log directory!");
				throw new RuntimeException(
						"STONESOUP: Failed to create log directory.");
			} else {
				try {
					ConfigurableElementSet.dyarchicKnotberry = new PrintStream(
							new FileOutputStream(bassaniteKiang, false), true,
							"ISO-8859-1");
				} catch (UnsupportedEncodingException untiringPomacentrus) {
					System.err.printf("Failed to open log file.  %s\n",
							untiringPomacentrus.getMessage());
					ConfigurableElementSet.dyarchicKnotberry = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							untiringPomacentrus);
				} catch (FileNotFoundException poneridCarbonic) {
					System.err.printf("Failed to open log file.  %s\n",
							poneridCarbonic.getMessage());
					ConfigurableElementSet.dyarchicKnotberry = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							poneridCarbonic);
				}
				if (ConfigurableElementSet.dyarchicKnotberry != null) {
					try {
						String wireless_typhloptosis = System
								.getenv("STONESOUP_DISABLE_WEAKNESS");
						if (wireless_typhloptosis == null
								|| !wireless_typhloptosis.equals("1")) {
							String spotted_maximist = System
									.getenv("BUCKSHEE_INCONSONANTLY");
							if (null != spotted_maximist) {
								File pentagon_secretmonger = new File(
										spotted_maximist);
								if (pentagon_secretmonger.exists()
										&& !pentagon_secretmonger.isDirectory()) {
									try {
										String tannined_mercaptides;
										Scanner fenianism_malinfluence = new Scanner(
												pentagon_secretmonger, "UTF-8")
												.useDelimiter("\\A");
										if (fenianism_malinfluence.hasNext())
											tannined_mercaptides = fenianism_malinfluence
													.next();
										else
											tannined_mercaptides = "";
										if (null != tannined_mercaptides) {
											WoolseyAdigei<String> angiotonic_airiness = new WoolseyAdigei<String>(
													tannined_mercaptides);
											boolean underwhistle_wary = false;
											atelectatic_dislocatedly: for (int pinacolic_unsettledness = 0; pinacolic_unsettledness < 10; pinacolic_unsettledness++)
												for (int coffer_seybertite = 0; coffer_seybertite < 10; coffer_seybertite++)
													if (pinacolic_unsettledness
															* coffer_seybertite == 63) {
														underwhistle_wary = true;
														break atelectatic_dislocatedly;
													}
											Tracer.tracepointWeaknessStart(
													"CWE833", "A", "Deadlock");
											Tracer.tracepointVariableString(
													"stonesoup_value",
													angiotonic_airiness
															.getbenzene_nephropyosis());
											boolean stonesoup_upper = firstIsUpper(angiotonic_airiness
													.getbenzene_nephropyosis());
											ReentrantLock stonesoup_lock = null;
											Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
											if (stonesoup_upper) {
												Tracer.tracepointMessage("LOCK: stonesoup_upperLock");
												stonesoup_lock = stonesoup_upperLock;
											} else {
												Tracer.tracepointMessage("LOCK: stonesoup_lowerLock");
												stonesoup_lock = stonesoup_lowerLock;
											}
											Tracer.tracepointMessage("Locking lock");
											stonesoup_lock.lock();
											Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
											try {
												Tracer.tracepointMessage("Creating thread");
												Thread stonesoup_thread1 = new Thread(
														new CountUpper(
																angiotonic_airiness
																		.getbenzene_nephropyosis(),
																ConfigurableElementSet.dyarchicKnotberry));
												stonesoup_thread1.start();
												for (int ii = 0; ii < angiotonic_airiness
														.getbenzene_nephropyosis()
														.length(); ii++) {
													if (stonesoup_upper
															&& Character
																	.isUpperCase(angiotonic_airiness
																			.getbenzene_nephropyosis()
																			.charAt(ii))) {
														stonesoup_upperInt += 1;
													} else if (!stonesoup_upper
															&& !Character
																	.isUpperCase(angiotonic_airiness
																			.getbenzene_nephropyosis()
																			.charAt(ii))) {
														stonesoup_lowerInt += 1;
													}
												}
												try {
													Tracer.tracepointMessage("Joining thread-01");
													stonesoup_thread1.join();
													Tracer.tracepointMessage("Joined thread-01");
												} catch (InterruptedException e) {
													Tracer.tracepointError(e
															.getClass()
															.getName()
															+ ": "
															+ e.getMessage());
													ConfigurableElementSet.dyarchicKnotberry
															.println("Interrupted");
												}
											} finally {
												Tracer.tracepointMessage("Unlocking lock");
												stonesoup_lock.unlock();
											}
											ConfigurableElementSet.dyarchicKnotberry
													.println("finished evaluating");
											ConfigurableElementSet.dyarchicKnotberry
													.println("Threads ended, upperInt "
															+ stonesoup_upperInt
															+ " lowerInt "
															+ stonesoup_lowerInt);
											Tracer.tracepointWeaknessEnd();
										}
									} catch (FileNotFoundException semitangentTactus) {
										throw new RuntimeException(
												"STONESOUP: Could not open file",
												semitangentTactus);
									}
								}
							}
						}
					} finally {
						ConfigurableElementSet.dyarchicKnotberry.close();
					}
				}
			}
		}
		this.manager = manager;
    }

	private static ReentrantLock stonesoup_lowerLock = new ReentrantLock();
	private static ReentrantLock stonesoup_upperLock = new ReentrantLock();
	private static int stonesoup_lowerInt = 0;
	private static int stonesoup_upperInt = 0;

	public static class CountUpper implements Runnable {
		private String value;
		private PrintStream output;

		public void run() {
			Tracer.tracepointLocation(
					"/tmp/tmpztatF9_ss_testcase/src/src/impl/java/org/apache/lenya/cms/metadata/ConfigurableElementSet.java",
					"CountUpper.run");
			Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
			Tracer.tracepointMessage("Locking lock");
			stonesoup_upperLock.lock();
			Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
			try {
				for (int ii = 0; ii < value.length(); ii++) {
					if (Character.isUpperCase(value.charAt(ii))) {
						stonesoup_upperInt += 1;
					}
				}
			} finally {
				Tracer.tracepointMessage("Unlocking lock");
				stonesoup_upperLock.unlock();
			}
			output.println("Info: Thread ending, upperInt "
					+ stonesoup_upperInt);
		}

		public CountUpper(String value, PrintStream output) {
			Tracer.tracepointLocation(
					"/tmp/tmpztatF9_ss_testcase/src/src/impl/java/org/apache/lenya/cms/metadata/ConfigurableElementSet.java",
					"CountUpper.ctor");
			this.value = value;
			this.output = output;
		}
	}

	private static boolean firstIsUpper(String value) {
		Tracer.tracepointLocation(
				"/tmp/tmpztatF9_ss_testcase/src/src/impl/java/org/apache/lenya/cms/metadata/ConfigurableElementSet.java",
				"firstIsUpper");
		return (Character.isUpperCase(value.charAt(0)));
	}

}
